﻿/**
* Main game class for episode 20 of One of those Weeks.
* Holds all the flags and data so different scenes all have access to all necessary information.
*
* @author Billy D. Spelchan
* @version 1.0
*/
class OotW20
{
	// ***** CONSTANTS *****
	public static var MESSAGEHIDE:Number = 1;
	public static var TILESET:String = "-+acmln";
	public static var TILE_EMPTY:Number = 0;
	public static var TILE_PLAYER:Number = 1;
	public static var TILE_STICK:Number = 2;
	public static var TILE_PIT:Number = 3;
	public static var TILE_ENTRANCE:Number = 4;
	public static var TILE_AXEHOLE:Number = 5;
	public static var TILE_WALL:Number = 6;
	
	// ***** VARIABLES *****
	
	public var haveStick:Boolean = false;
	public var seenPit:Boolean = false;
	public var haveAxe:Boolean = false;
	public var noPitPickup:Boolean = false;
	public var lightCount:Number = 256;
	
	public var RAW_MAP:Array;
	public var dirString:Array;
	public var tdTilemap:Array;
	public var automapData:Array;
	public var playerX:Number = 1;
	public var playerY:Number = 30;
	public var playerD:Number = 2;
	
	// ***** CONSTRUCTOR *****
	public function OotW20()
	{
		RAW_MAP = new Array(
//1234567890123456789012345678901
"nnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnn",
"n--n-----n------c--nn----n----nn",
"n-nnnn--nn--nnnnnn--n-n--n-nn--n",
"n-n-n--nn--nn----n--n-na-n--n--n",
"n-n---nn--nnnn-n-nn-n-nnnn--nn+m",
"n-nnn-n--nn-n--n--n--------nn--n",
"n-----n-nn--c-nn-nnnnn-nn-nn---n",
"nnnn-nn-n---nnn--n---nnn-----n-n",
"nn------n-nnn-n--nn------n-n-n-n",
"n--nn--nn-n---nn----nn---n-n-n-n",
"n-nn--nn--n-n--nnnnnn--nnn-n-n-n",
"nnn--nn--nn-n---------nn---n-n-n",
"n---nn----n-n-nnn-nnnnn--nnn-ncn",
"n--nn--n-nn-nnn-nnn-----nn---n-n",
"n-nn--nnnn---n-------nnnn--n-n-n",
"n-nnnnnn---n-nn---nnnn----nn-n-n",
"n--nn--c-n-n--nnn----nnnnnnn-n-n",
"nnnn--nn---n----nn----n---c--n-n",
"n-----n--nnn-n---nnnn---nnnn-n-n",
"n-nn-nn-nn---n-n----nnnnn--n-n-n",
"nnn--n---n-nnn-nnnn--------n-n-n",
"nnnnnnnnnnnn-n----nnn--nnn-n-n-n",
"nn-----------nnnn---nnnnn--n-n-n",
"nn-nnnnnnnnn-n--nnnnnn----nn-n-n",
"nn---------nnn-----------nn--n-n",
"nnnnnnnnn----n-nnnnnn---nn--nn-n",
"n-n-----nnn-nnnn--n---nnn--nn--n",
"n----nn-----nn------nnn---nnn--n",
"n-nn-nnnnn---c--nnnnn---nnnnn-nn",
"n--nnn---nnnnnnnn-----nnnnn---nn",
"n----n---------c---nnnn-----nnnn",
"nnnnlnnnnnnnnnnnnnnnnnnnnnnnnnnn"
//1234567890123456789012345678901
						);
		dirString = new Array( "North", "East", "South", "West");
		tdTilemap = new Array(32);
		automapData = new Array;
		for (var cntr:Number = 0; cntr < 32; ++cntr)
			automapData[cntr] = new Array(32);
	}
	
	public function restartGame()
	{
		var cntr:Number;
		
		haveStick = false;
		seenPit = false;
		haveAxe = false;
		noPitPickup = false;
		lightCount = 256;

		playerX = 30;
		playerY = 4;
		playerD = 3;
		for (cntr = 0; cntr < 32; ++cntr)
			tdTilemap[cntr] = RAW_MAP[cntr];
		for (var cntrY:Number = 0; cntrY < 32; ++cntrY)
			for (var cntrX:Number = 0; cntrX < 32; ++cntrX)
				automapData[cntrY][cntrX] = 0;
	}
	
	function getOffsetTile(dist:Number, lr:Number):Number
	{
		var tx:Number = playerX;
		var ty:Number = playerY;
		var lradj:Number = lr;
		switch (playerD)
		{
			case 0: // north
				tx += lradj;
				ty -= dist;
				break;
			case 1: // east
				tx += dist;
				ty += lradj;
				break;
			case 2: // south
				tx -= lradj;
				ty += dist;
				break;
			case 3: // west
				tx -= dist;
				ty -= lradj;
				break;
		}
		while(tx < 0) tx += 32;
		while(tx > 31) tx -= 32;
		while (ty < 0) ty += 32;
		if (ty > 31)
			return TILE_WALL;
	//	while (ty > 31) ty -= 32;
		var tchar:String = tdTilemap[ty].charAt(tx);
		var tval:Number = TILESET.indexOf(tchar);
		if ( (tval == TILE_STICK) && (haveStick == true))
			tval = TILE_EMPTY;
		return tval;
	}

	function canMoveForward()
	{
		var canEnter:Boolean = true;
		var tileType:Number = getOffsetTile(1, 0);
		switch (tileType) {
			case TILE_EMPTY:
			case TILE_PLAYER:
			case TILE_STICK:
				break; // already true
			case TILE_PIT:
				canEnter = haveStick;
				if (lightCount < 1) 
					canEnter = true; // will fall into pits in the dark
				break;
			case TILE_ENTRANCE:
				canEnter = haveAxe;
				break;
			case TILE_AXEHOLE:
			case TILE_WALL:
				canEnter = false;
				break;
		}
		return canEnter;
	}
	
	function enterTile(nx:Number, ny:Number):Number
	{		
		var id:Number = getOffsetTile(0,0);
		var tempTile:Number;
		
		lightCount = Math.max(lightCount - 1, 0);
		if (lightCount <= 0 )
			return id;
		var tx:Number, ty:Number;
		var tchar:String;
		for (var cntrX:Number = -3; cntrX < 4; ++cntrX) {
			tx = nx+cntrX;
			if (tx < 0) continue;
			if (tx > 31) break;
			for (var cntrY:Number = -3; cntrY < 4; ++cntrY) {
				ty = (ny+cntrY);
				if (ty < 0) continue;
				if (ty > 31) break;
				tchar = tdTilemap[ty].charAt(tx);
				tempTile = TILESET.indexOf(tchar);
				automapData[ny+cntrY][nx+cntrX] = tempTile + 1; // remember 0 is used for unseen tile
			}
		}
		return id;
	}
	
	function getMaxScore():Number
	{
		return 20;
	}
	
	function getFinalScore():Number
	{
		// 5 points for finding axe
		// 5 points for finding log
		var score:Number = 10;
		
		// 5 points for knowing need log (seeing pits) before taking log
		if (noPitPickup == false)
			score += 5;
			
		// 5 points for leaving cave with light left
		if (lightCount > 0)
			score += 5;

		return score;
	}
}